"""OAuthUserRepository class for the OAuth user."""

from __future__ import annotations

import datetime
import logging
from typing import List

import pymongo
from bson import ObjectId
from bson.errors import InvalidId
from motor.motor_asyncio import AsyncIOMotorCollection
from pymongo.errors import DuplicateKeyError
from pymongo.results import InsertOneResult

from zmp_authentication_provider.exceptions import (
    InvalidObjectIDException,
    ObjectNotFoundException,
)
from zmp_authentication_provider.scheme.auth_model import OAuthUser

log = logging.getLogger(__name__)

DEFAULT_TIME_FORMAT = "%Y-%m-%dT%H:%M:%S%z"
"""Default time format and time zone %Y-%m-%dT%H:%M:%S%z"""
DEFAULT_TIME_ZONE = datetime.UTC
"""Default time zone is UTC(+00:00)"""


class OAuthUserRepository:
    """OAuthUserRepository class."""

    def __init__(self, *, collection: AsyncIOMotorCollection):
        """Initialize the repository with MongoDB database."""
        self._collection = collection

    @classmethod
    async def create(cls, *, collection: AsyncIOMotorCollection) -> OAuthUserRepository:
        """Create a new instance of the repository."""
        instance = cls(collection=collection)
        await instance.init_index()
        return instance

    async def init_index(self) -> None:
        """Create indexes for the collection."""
        indexes = await self._collection.list_indexes().to_list(length=None)

        unique_index_name = "unique_key_sub_iss"
        email_index_name = "index_email"
        username_index_name = "index_username"

        unique_index_exists = False
        email_index_exists = False
        username_index_exists = False

        for index in indexes:
            if index["name"] == unique_index_name and index.get("unique", True):
                unique_index_exists = True
            if index["name"] == email_index_name:
                email_index_exists = True
            if index["name"] == username_index_name:
                username_index_exists = True

        if not unique_index_exists:
            await self._collection.create_index(
                [("sub", 1), ("iss", 1)], name=unique_index_name, unique=True
            )

        if not email_index_exists:
            await self._collection.create_index("email", name=email_index_name)

        if not username_index_exists:
            await self._collection.create_index("username", name=username_index_name)

        return self

    async def insert(self, oauth_user: OAuthUser) -> str:
        """Insert an OAuth user."""
        if oauth_user.created_at is None:
            oauth_user.created_at = datetime.datetime.now(DEFAULT_TIME_ZONE)

        oauth_user_dict = oauth_user.model_dump(by_alias=True, exclude=["id"])
        oauth_user_dict.update({"created_at": oauth_user.created_at})

        try:
            result: InsertOneResult = await self._collection.insert_one(oauth_user_dict)
        except DuplicateKeyError as e:
            raise ValueError(f"OAuthUser already exists: {e}")

        return str(result.inserted_id)

    async def find(self) -> List[OAuthUser]:
        """Find all OAuth users."""
        cursor = self._collection.find().sort("created_at", pymongo.DESCENDING)
        oauth_users = []
        async for document in cursor:
            if document is not None:
                oauth_user = OAuthUser(**document)
                oauth_users.append(oauth_user)

        return oauth_users

    async def update(self, oauth_user: OAuthUser) -> OAuthUser:
        """Update an OAuth user."""
        try:
            query = {"_id": ObjectId(oauth_user.id)}
        except InvalidId as e:
            raise InvalidObjectIDException(e)

        if oauth_user.updated_at is None:
            oauth_user.updated_at = datetime.datetime.now(DEFAULT_TIME_ZONE)

        oauth_user_dict = oauth_user.model_dump(
            by_alias=True, exclude=["id", "created_at"]
        )
        oauth_user_dict.update({"updated_at": oauth_user.updated_at})
        update = {"$set": oauth_user_dict}

        log.debug(f"Update oauth_user: {oauth_user.id}")
        log.debug(f"Update data: {update}")

        document = await self._collection.find_one_and_update(
            query, update=update, return_document=pymongo.ReturnDocument.AFTER
        )

        if document is None:
            raise ObjectNotFoundException(object_id=oauth_user.id)

        updated = OAuthUser(**document)
        return updated

    async def find_by_id(self, id: str) -> OAuthUser:
        """Find an OAuth user by id."""
        if not id:
            raise InvalidObjectIDException("OAuthUser id is required")

        try:
            query = {"_id": ObjectId(id)}
        except InvalidId as e:
            raise InvalidObjectIDException(e)

        document = await self._collection.find_one(query)

        if document is None:
            raise ObjectNotFoundException(object_id=id)

        oauth_user = OAuthUser(**document)
        return oauth_user

    async def find_by_sub(self, sub: str) -> OAuthUser:
        """Find an OAuth user by sub field."""
        if not sub:
            raise InvalidObjectIDException("OAuthUser sub is required")

        document = await self._collection.find_one({"sub": sub})

        if document is None:
            raise ObjectNotFoundException(object_id=sub)

        oauth_user = OAuthUser(**document)
        return oauth_user

    async def find_by_username(self, username: str) -> OAuthUser:
        """Find an OAuth user by username field."""
        if not username:
            raise InvalidObjectIDException("OAuthUser username is required")

        document = await self._collection.find_one({"username": username})

        if document is None:
            raise ObjectNotFoundException(object_id=username)

        oauth_user = OAuthUser(**document)
        return oauth_user

    async def find_by_email(self, email: str) -> OAuthUser:
        """Find an OAuth user by email."""
        if not email:
            raise InvalidObjectIDException("OAuthUser email is required")

        document = await self._collection.find_one({"email": email})

        if document is None:
            raise ObjectNotFoundException(object_id=email)

        oauth_user = OAuthUser(**document)
        return oauth_user

    async def find_by_sub_and_issuer(self, sub: str, iss: str) -> OAuthUser:
        """Find an OAuth user by sub and issuer."""
        if not sub:
            raise InvalidObjectIDException("OAuthUser sub is required")
        if not iss:
            raise InvalidObjectIDException("OAuthUser issuer is required")

        document = await self._collection.find_one({"sub": sub, "iss": iss})

        if document is None:
            raise ObjectNotFoundException(object_id=f"{sub}@{iss}")

        oauth_user = OAuthUser(**document)
        return oauth_user

    async def delete_by_id(self, oauth_user_id: str) -> bool:
        """Delete an OAuth user by id."""
        if not oauth_user_id:
            raise InvalidObjectIDException("OAuthUser id is required")

        try:
            query = {"_id": ObjectId(oauth_user_id)}
        except InvalidId as e:
            raise InvalidObjectIDException(e)

        result = await self._collection.find_one_and_delete(query)

        if result is None:
            raise ObjectNotFoundException(f"OAuthUser not found: {oauth_user_id}")

        return True

    async def delete_by_sub(self, sub: str) -> bool:
        """Delete an OAuth user by sub."""
        if not sub:
            raise InvalidObjectIDException("OAuthUser sub is required")

        result = await self._collection.find_one_and_delete({"sub": sub})

        if result is None:
            raise ObjectNotFoundException(f"OAuthUser not found: {sub}")

        return True



"""OAuth2 Keycloak module for the AIops Pilot."""

import logging
from typing import Any

from fastapi import HTTPException, Request, status

log = logging.getLogger(__name__)

USER_SESSION_KEY = "user_info"


async def get_current_user_in_session(request: Request) -> dict[str, Any]:
    """Get the user info from the request."""
    user_info: dict[str, Any] = request.session.get(USER_SESSION_KEY)
    if not user_info:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="User info is not found in the request session",
        )

    return user_info



"""InMemorySessionStore is a simple in-memory session store for the authentication provider."""

import logging
import time
from datetime import datetime, timedelta
from threading import Lock, Thread
from typing import Any

from pydantic import BaseModel

from zmp_authentication_provider.utils.time_utils import DEFAULT_TIME_ZONE

logger = logging.getLogger(__name__)

class SessionData(BaseModel):
    """SessionData is a model for the session data."""

    user_info: dict[str, Any]
    access_token: str
    refresh_token: str
    id_token: str
    expires_at: datetime | None = None


class InMemorySessionStore:
    """InMemorySessionStore is a simple in-memory session store for the authentication provider."""

    def __init__(self, session_ttl: int = 3600, session_cleanup_interval: int = 300):
        """Initialize the InMemorySessionStore."""
        self.session_store: dict[str, SessionData] = {}
        self.session_ttl = session_ttl
        self.session_cleanup_interval = session_cleanup_interval
        self.session_store_lock = Lock()
        self._running = True
        self.session_cleanup_thread = Thread(target=self.session_cleanup, daemon=True)
        self.session_cleanup_thread.start()

    def get(self, unique_id: str) -> SessionData:
        """Get the session data from the in-memory session store."""
        return self.session_store.get(unique_id)

    def set(self, unique_id: str, session_data: SessionData):
        """Set the session data in the in-memory session store."""
        # delete the session data if it exists
        if self.get(unique_id):
            self.delete(unique_id)

        # set the expires_at
        session_data.expires_at = datetime.now(DEFAULT_TIME_ZONE) + timedelta(seconds=self.session_ttl)

        # set the session data in the in-memory session store
        self.session_store[unique_id] = session_data

    def reset_ttl(self, unique_id: str):
        """Reset the ttl of the session data in the in-memory session store."""
        session_data = self.get(unique_id)
        if session_data:
            session_data.expires_at = datetime.now(DEFAULT_TIME_ZONE) + timedelta(seconds=self.session_ttl)
            self.session_store[unique_id] = session_data
            logger.info(f"Session data for {unique_id} has been reset ttl until {session_data.expires_at}")

    def delete(self, unique_id: str):
        """Delete the session data from the in-memory session store."""
        if self.get(unique_id):
            del self.session_store[unique_id]
        else:
            # skip if the session data is not found in the in-memory session store
            pass

    def session_cleanup(self):
        """Cleanup the session data from the in-memory session store."""
        while self._running:
            with self.session_store_lock:
                for unique_id, session_data in self.session_store.items():
                    if session_data.expires_at and session_data.expires_at < datetime.now(DEFAULT_TIME_ZONE):
                        self.delete(unique_id)
                        logger.info(f"Session data for {unique_id} has expired and has been deleted.")

                logger.info(f"Session cleanup completed. {len(self.session_store)} sessions remaining.")

            time.sleep(self.session_cleanup_interval)

    def close(self):
        """Explicitly clean up resources."""
        self._running = False
        if self.session_cleanup_thread.is_alive():
            self.session_cleanup_thread.join(timeout=5)


"""Auth routes."""

import base64
import json
import logging
from typing import Any

import requests
from fastapi import APIRouter, Depends, HTTPException, Request, status
from fastapi.responses import HTMLResponse, RedirectResponse

from zmp_authentication_provider.auth.oauth2_keycloak import (
    KEYCLOAK_AUTH_ENDPOINT,
    KEYCLOAK_CLIENT_ID,
    KEYCLOAK_CLIENT_SECRET,
    KEYCLOAK_END_SESSION_ENDPOINT,
    KEYCLOAK_REDIRECT_URI,
    KEYCLOAK_TOKEN_ENDPOINT,
    KEYCLOAK_USER_ENDPOINT,
    TokenData,
    get_current_user,
    verify_token,
)
from zmp_authentication_provider.auth.session_auth import (
    USER_SESSION_KEY,
    get_current_user_in_session,
)
from zmp_authentication_provider.exceptions import (
    AuthBackendException,
    AuthError,
)
from zmp_authentication_provider.routes import get_auth_service
from zmp_authentication_provider.scheme.auth_model import OAuthUser
from zmp_authentication_provider.service.auth_service import AuthService
from zmp_authentication_provider.setting import auth_default_settings
from zmp_authentication_provider.utils.session_memory import (
    InMemorySessionStore,
    SessionData,
)

logger = logging.getLogger(__name__)


in_memory_session_store = InMemorySessionStore(
    session_ttl=auth_default_settings.session_ttl,
    session_cleanup_interval=auth_default_settings.session_cleanup_interval,
)

router = APIRouter(
    prefix="/auth",
    tags=["auth"],
)


@router.get("/home", summary="Home page", response_class=HTMLResponse)
async def home(request: Request):  # , csrf_token: str = Depends(csrf_scheme)):
    """Get home page."""
    # check the session_id in the cookie
    csrf_token = request.cookies.get(auth_default_settings.csrf_token_cookie_name)
    session_id = request.cookies.get(auth_default_settings.session_id_cookie_name)

    logger.debug(f"csrf_token: {csrf_token}")
    logger.debug(f"session_id: {session_id}")

    if csrf_token and session_id:
        user_info = request.session.get("user_info")
        if not user_info:
            request.session.clear()
            return HTMLResponse(
                content="Session data has been lost "
                "because the server has been restared."
                "Please login again",
            )
        else:
            return HTMLResponse(content=f"<p>User info</p><p>{user_info}</p>")
    else:
        return HTMLResponse(content="<p>No user info. Please login again.</p>")


@router.get(
    "/authenticate",
    summary="Process the authentication",
    response_class=RedirectResponse,
)
async def authenticate(request: Request):
    """Authenticate whether the user is logged in or not."""
    # check the session_id in the cookie
    csrf_token = request.cookies.get(auth_default_settings.csrf_token_cookie_name)
    session_id = request.cookies.get(auth_default_settings.session_id_cookie_name)

    logger.debug(f"csrf_token: {csrf_token}")
    logger.debug(f"session_id: {session_id}")

    # for the redirect to the referer
    referer = request.headers.get("referer")
    # referer = referer or "/"
    if not referer:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details="Referer is not found in the request header",
        )

    logger.debug(f"referer: {referer}")

    if csrf_token and session_id:
        access_token = request.session.get("access_token")
        if not access_token:
            request.session.clear()
            logger.error(
                "Session data has been lost "
                "because the server has been restared."
                "Please login again",
            )
            return RedirectResponse(
                url=f"{KEYCLOAK_AUTH_ENDPOINT}?response_type=code"
                f"&client_id={KEYCLOAK_CLIENT_ID}"
                f"&state={csrf_token}{auth_default_settings.state_separator}{referer}"
                f"&redirect_uri={KEYCLOAK_REDIRECT_URI}"
                f"&scope=openid profile email"
                # NOTE: referer is not used in the keycloak
                # f"&referer={referer}"
            )
        else:
            logger.debug(f"access_token: {access_token[:100]}...")

            return RedirectResponse(url=f"{referer}")
    else:
        return RedirectResponse(
            url=f"{KEYCLOAK_AUTH_ENDPOINT}?response_type=code"
            f"&client_id={KEYCLOAK_CLIENT_ID}"
            f"&state={csrf_token}{auth_default_settings.state_separator}{referer}"
            f"&redirect_uri={KEYCLOAK_REDIRECT_URI}"
            f"&scope=openid profile email"
            # NOTE: referer is not used in the keycloak
            # f"&referer={referer}"
        )


@router.get(
    "/logout",
    summary="Logout from the keyclaok",
    response_class=RedirectResponse,
)
async def logout(
    request: Request,
    # csrf_token: str = Depends(csrf_scheme),
    # session_id: str = Depends(session_id_scheme)
):
    """Logout."""
    csrf_token = request.cookies.get(auth_default_settings.csrf_token_cookie_name)
    session_id = request.cookies.get(auth_default_settings.session_id_cookie_name)
    if not csrf_token or not session_id:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="No csrf token in cookie and session id in cookie",
        )
    else:
        user_info = request.session.get("user_info")
        if not user_info:
            raise AuthBackendException(
                AuthError.OAUTH_IDP_ERROR,
                details="User info is not found in the request session",
            )
        user_id = user_info.get("sub")
        if not user_id:
            raise AuthBackendException(
                AuthError.OAUTH_IDP_ERROR,
                details="User ID is not found in the user info",
            )
        session_data = in_memory_session_store.get(user_id)
        if not session_data:
            raise AuthBackendException(
                AuthError.OAUTH_IDP_ERROR,
                details="Session data is not found in the in-memory session store",
            )

        # refresh_token = request.session.get("refresh_token")
        refresh_token = session_data.refresh_token
        if not refresh_token:
            raise AuthBackendException(
                AuthError.OAUTH_IDP_ERROR,
                details="Refresh token is not found in the session data",
            )
        else:
            # 1. logout from the keycloak
            data = {
                "client_id": KEYCLOAK_CLIENT_ID,
                "client_secret": KEYCLOAK_CLIENT_SECRET,
                "refresh_token": refresh_token,
            }
            headers = {"Content-Type": "application/x-www-form-urlencoded"}
            idp_response = requests.post(
                KEYCLOAK_END_SESSION_ENDPOINT,
                data=data,
                headers=headers,
                verify=auth_default_settings.http_client_ssl_verify,
            )  # verify=False: because of the SKCC self-signed certificate

            if idp_response.status_code != 204:
                raise AuthBackendException(
                    AuthError.OAUTH_IDP_ERROR,
                    details=f"Failed to logout.({idp_response.reason})",
                )
        # 2. delete the session data from the in-memory session store
        in_memory_session_store.delete(user_id)
        # 3. clear the session
        request.session.clear()

        # NOTE: go to the authenticate endpoint with the referer (/) after logout
        redirect_response = RedirectResponse(
            url=f"{auth_default_settings.application_endpoint}/auth/authenticate",
            headers={"referer": "/"},
        )
        # clear the csrf token cookie
        # NOTE: the csrf token cookie should be kept for the next request
        # redirect_response.delete_cookie(
        #     key=auth_default_settings.csrf_token_cookie_name
        # )

        return redirect_response


@router.get("/oauth2/callback", summary="Keycloak OAuth2 callback for the redirect URI")
async def callback(
    request: Request,
    code: str,
    state: str,
    auth_service: AuthService = Depends(get_auth_service),
):
    """Keycloak OAuth2 callback for the redirect URI."""
    csrf_token = request.cookies.get(auth_default_settings.csrf_token_cookie_name)

    state = state.split(auth_default_settings.state_separator)
    received_csrf_token = state[0]
    referer = state[1]

    logger.debug(f"cookie csrftoken: {csrf_token}")
    logger.debug(f"state: {state}")
    logger.debug(f"received csrftoken: {received_csrf_token}")
    logger.debug(f"referer: {referer}")

    if received_csrf_token != csrf_token:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details=f"CSRF token mismatch.({received_csrf_token} != {csrf_token})",
        )

    data = {
        "grant_type": "authorization_code",
        "code": code,
        "redirect_uri": KEYCLOAK_REDIRECT_URI,
        "client_id": KEYCLOAK_CLIENT_ID,
        "client_secret": KEYCLOAK_CLIENT_SECRET,
    }
    headers = {
        "Content-Type": "application/x-www-form-urlencoded",
        "Accept": "application/json",
    }
    idp_response = requests.post(
        KEYCLOAK_TOKEN_ENDPOINT,
        data=data,
        headers=headers,
        verify=auth_default_settings.http_client_ssl_verify,
    )

    if idp_response.status_code != 200:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details=f"Failed to obtain token.({idp_response.reason})",
        )

    tokens = idp_response.json()

    access_token = tokens.get("access_token")
    refresh_token = tokens.get("refresh_token")
    id_token = tokens.get("id_token")

    # NOTE: This is not needed anymore because the access token is used for the authentication
    # -----------------------------------------------------------------------------------------
    headers = {"Authorization": f"Bearer {access_token}", "Accept": "application/json"}
    idp_response = requests.get(
        KEYCLOAK_USER_ENDPOINT,
        headers=headers,
        verify=auth_default_settings.http_client_ssl_verify,
    )
    if idp_response.status_code != 200:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details=f"Failed to fetch user info.({idp_response.reason})",
        )

    user_info = idp_response.json()
    logger.debug(f"user_info: {user_info}")
    user_id = user_info.get("sub")
    if not user_id:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details="User ID is not found in the user info",
        )

    # -----------------------------------------------------------------------------------------
    # NOTE: the user_info is only used for the authentication
    request.session[USER_SESSION_KEY] = user_info

    # check the session data size because the max size of the cookie is 4kb
    json_session = json.dumps(request.session)
    logger.debug(f"json_session: {json_session}")
    logger.debug(f"json_session bytes: {len(json_session.encode('utf-8'))}")

    base64_encoded_session = base64.b64encode(json_session.encode("utf-8"))
    logger.debug(f"base64_encoded_session: {base64_encoded_session}")

    total_bytes = len(base64_encoded_session)
    logger.debug(f"base64_encoded_session bytes: {total_bytes}")

    if total_bytes > (4 * 1024):
        logger.debug(f"Total bytes: {total_bytes}")
        logger.warning(f"The session data size({total_bytes}) is over than 4kb.")
        raise AuthBackendException(
            AuthError.TOKEN_DATA_TOO_LARGE,
            details=f"The session data size is {total_bytes} bytes. It is over than 4kb.",
        )

    # store the session data in the in-memory session store
    session_data = SessionData(
        user_info=user_info,
        access_token=access_token,
        refresh_token=refresh_token,
        id_token=id_token,
    )
    in_memory_session_store.set(user_id, session_data)

    # -----------------------------------------------------------------------------------------
    # create the oauth user if the enable_oauth_user_creation is True
    if auth_default_settings.enable_oauth_user_creation:
        # NOTE: the userinfo does not have the iss, so we need to verify the id_token for OAuthUser creation
        token_data = verify_token(id_token)
        oauth_user = OAuthUser(
            iss=token_data.iss,
            sub=token_data.sub,
            username=token_data.username,
            email=token_data.email,
            given_name=token_data.given_name,
            family_name=token_data.family_name,
        )

        logger.debug(f"oauth_user: {oauth_user}")

        await auth_service.upsert_oauth_user(oauth_user)

    # If the same-site of cookie is 'lax', the cookie will be sent only if the request is same-site request
    # If the same-site of cookie is 'strict', the cookie will not be sent
    return RedirectResponse(
        # url=f"{auth_default_settings.application_endpoint}/auth/home"
        url=f"{referer}"
    )


@router.get(
    "/access_token", summary="Get the access token from the in-memory session store"
)
async def get_access_token(
    user_info: dict[str, Any] = Depends(get_current_user_in_session),
):
    """Get the access token from the in-memory session store."""
    user_id = user_info.get("sub")
    if not user_id:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details="User ID is not found in the user info",
        )

    session_data = in_memory_session_store.get(user_id)
    if not session_data:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details="Session data is not found in the in-memory session store",
        )
    return {"access_token": session_data.access_token}


@router.patch(
    "/refresh_token", summary="Refresh the access token using the refresh token"
)
async def refresh_access_token(
    user_info: dict[str, Any] = Depends(get_current_user_in_session),
):
    """Refresh the access token using the refresh token."""
    user_id = user_info.get("sub")
    if not user_id:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details="User ID is not found in the user info",
        )

    session_data = in_memory_session_store.get(user_id)
    if not session_data:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details="Session data is not found in the in-memory session store",
        )

    refresh_token = session_data.refresh_token
    if not refresh_token:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details="Refresh token is not found in the session data",
        )

    data = {
        "grant_type": "refresh_token",
        "refresh_token": refresh_token,
        "client_id": KEYCLOAK_CLIENT_ID,
        "client_secret": KEYCLOAK_CLIENT_SECRET,
    }
    headers = {
        "Content-Type": "application/x-www-form-urlencoded",
        "Accept": "application/json",
    }
    idp_response = requests.post(
        KEYCLOAK_TOKEN_ENDPOINT,
        data=data,
        headers=headers,
        verify=auth_default_settings.http_client_ssl_verify,
    )

    if idp_response.status_code != 200:
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details=f"Failed to refresh the access token.({idp_response.reason})",
        )

    refreshed_tokens = idp_response.json()
    # TODO: check the tokens whether the access token, refresh token, and id token are included in the tokens
    logger.debug(f"tokens: {refreshed_tokens}")

    if not refreshed_tokens.get("access_token"):
        raise AuthBackendException(
            AuthError.OAUTH_IDP_ERROR,
            details="Access token is not found in the response",
        )
    else:
        # update the session data in the in-memory session store with the new tokens
        session_data = SessionData(
            user_info=user_info,
            access_token=refreshed_tokens.get("access_token"),
            refresh_token=refreshed_tokens.get("refresh_token"),
            id_token=refreshed_tokens.get("id_token"),
        )
        in_memory_session_store.set(user_id, session_data)

        return {"access_token": session_data.access_token}


@router.get("/profile", summary="Get the current user profile from Token")
async def profile(oauth_user: TokenData = Depends(get_current_user)):
    """Get the current user profile from Token."""
    session_data = in_memory_session_store.get(oauth_user.sub)
    if not session_data:
        raise AuthBackendException(
            AuthError.SESSION_EXPIRED,
            details="Session data is not found in the in-memory session store",
        )
    in_memory_session_store.reset_ttl(oauth_user.sub)

    return oauth_user
